import { styled } from "solid-styled-components";
import { vector2d } from "../vector";
import { createMemo, createSignal, onCleanup, onMount } from "solid-js";

import "./crystal.css";

export default function Crystal(props: { size?: number; position?: vector2d }) {
  const size = props.size ?? 100;

  // Position

  const [x] = createSignal(
    props.position?.x ? props.position?.x - size / 2 : 0
  );
  const [y] = createSignal(
    props.position?.y ? props.position?.y - size / 2 : 0
  );

  // Energy

  const [energy, setEnergy] = createSignal(100);
  const [maxEnergy, setMaxEnergy] = createSignal(100);

  const rechargeProgress = createMemo(() => (energy() / maxEnergy()) * 100);

  function updateEnergy() {
    setEnergy(energy() - 1);
  }

  onMount(() => {
    const loop = setInterval(() => updateEnergy(), 100);
    onCleanup(() => clearInterval(loop));
  });

  // Renderer

  const renderRandomFactor = Math.random();

  const CrystalElement = styled("svg")`
    width: ${size.toString()}px;
    height: ${size.toString()}px;

    background-color: transparent;
    position: absolute;

    --half-size: calc(var(--size) / 2);
    --stroke-width: 8px;
    --radius: calc(
      (var(--size) - var(--stroke-width)) / 2 - var(--stroke-width) / 2
    );
    --circumference: calc(var(--radius) * pi * 2);
    --dash: calc((var(--progress) * var(--circumference)) / 100);
  `;

  const CrystalCircleActive = styled("circle")`
    cx: var(--half-size);
    cy: var(--half-size);
    r: var(--radius);
    stroke-width: var(--stroke-width);
    fill: none;
    stroke-linecap: round;

    animation: alternate 10s crystal--fill-wabble infinite ease-in-out;
    transform-origin: var(--half-size) var(--half-size);
    stroke-dasharray: var(--dash) calc(var(--circumference) - var(--dash));
    transition: stroke-dasharray 0.3s linear 0s;
    stroke: var(--color-primary);
  `;

  const CrystalCircleInactive = styled("circle")`
    cx: var(--half-size);
    cy: var(--half-size);
    r: var(--radius);
    stroke-width: var(--stroke-width);
    fill: none;
    stroke-linecap: round;

    stroke: #ddd;
  `;

  return (
    <CrystalElement
      class="crystal"
      style={{
        left: `${x()}px`,
        top: `${y()}px`,

        "--size": `${size}px`,
        "--rotation-factor": renderRandomFactor * 10,
        "--progress": rechargeProgress(),
      }}
    >
      <CrystalCircleInactive />
      <CrystalCircleActive />
    </CrystalElement>
  );
}
